# pymonday/methods/updates.py

########################################################################################################################
# UPDATES CLASS
########################################################################################################################
class Updates:
    def __init__(self, client):
        self.client = client

    ####################################################################################################################
    def get_item_updates(self, item_id):
        """
        Get the updates from an item.
        :param item_id: UUID of the item.
        :return: Array of Dictionaries containing the update data. Return None or error message on API call failure.
        """
        query_string = f'''{{items(ids:{item_id})
        {{updates {{id text_body, updated_at created_at creator_id 
        replies {{text_body created_at creator_id}} 
        assets {{id public_url}}}}}}}}'''
        data = {'query': query_string}
        response = self.client.send_post_request(data)
        if not response:
            return None
        try:
            return response['data']['items'][0]['updates']
        except (KeyError, IndexError, TypeError):
            return response

    ####################################################################################################################
    def create_update(self, item_id, update_text):
        """
        Create an update on an item.
        :param item_id: UUID of the item to leave the update on.
        :param update_text: Body of the update.
        :return: UUID of the update. Return None or error message on API call failure.
        """
        query_string = f'''mutation {{create_update (item_id: {item_id}, body: "{update_text}") {{id}}}}'''
        data = {'query': query_string}
        response = self.client.send_post_request(data)
        if not response:
            return None
        try:
            return response['data']['create_update']['id']
        except (KeyError, IndexError, TypeError):
            return response

    ####################################################################################################################
    def create_reply(self, item_id, update_text, parent_id):
        """
        Create an update on an item.
        :param item_id: UUID of the item to leave the update on.
        :param update_text: Body of the update.
        :param parent_id: UUID of the update to leave a reply on.
        :return: UUID of the Reply. Return None or error message on API call failure.
        """
        query_string = f'''mutation {{
        create_update (item_id: {item_id}, body: "{update_text}", parent_id: {parent_id}) {{id}}}}'''
        data = {'query': query_string}
        response = self.client.send_post_request(data)
        if not response:
            return None
        try:
            return response['data']['create_update']['id']
        except (KeyError, IndexError, TypeError):
            return response

    ####################################################################################################################
    def delete_update(self, update_id):
        """
        Delete an update on an item.
        :param update_id: UUID of the item.
        :return: True on success. Return None or error message on API call failure.
        """
        query_string = f'''mutation {{delete_update (id: {update_id}) {{id}}}}'''
        data = {'query': query_string}
        response = self.client.send_post_request(data)
        if not response:
            return None
        try:
            if response['data']['delete_update']['id']:
                return True
            else:
                return response
        except (TypeError, KeyError, IndexError):
            return response

    ####################################################################################################################
    def clear_updates(self, item_id):
        """
        Clear all updates from an item.
        :param item_id: UUID of the item.
        :return: True on success. Return None or error message on API call failure.
        """
        query_string = f'''mutation {{clear_item_updates (item_id: {item_id}) {{id}}}}'''
        data = {'query': query_string}
        response = self.client.send_post_request(data)
        if not response:
            return None
        try:
            if response['data']['clear_item_updates']['id']:
                return True
            else:
                return response
        except (TypeError, KeyError, IndexError):
            return response

    ####################################################################################################################


# pymonday/methods/groups.py

########################################################################################################################
# GROUPS CLASS
########################################################################################################################
class Groups:
    def __init__(self, client):
        self.client = client

    ####################################################################################################################
    def create_group(self, board_id, group_name):
        """
        Create a new group on a board at the top position.
        :param board_id: UUID of the board to create the item on.
        :param group_name: Name of the new group to be created.
        :return: UUID of the newly created group. Return None or error message on API call failure.
        """
        query_string = f'''mutation {{create_group (board_id: {board_id}, group_name: "{group_name}") {{id}}}}'''
        data = {'query': query_string}
        response = self.client.send_post_request(data)

        # Handle API response
        if not response:
            return None

        try:
            return response['data']['create_group']['id']
        except (TypeError, KeyError, IndexError):
            return response

    ####################################################################################################################
    def delete_group(self, board_id, group_id):
        """
        Delete a group from a board.
        :param board_id: UUID of the board to delete the group from.
        :param group_id: UUID of the group to delete.
        :return: True if successful. Return None or error message on API call failure.
        """
        query_string = f'''mutation {{delete_group (board_id: {board_id}, group_id: "{group_id}") {{id}}}}'''
        data = {'query': query_string}
        response = self.client.send_post_request(data)

        # Handle API response
        if not response:
            return None
        try:
            if response['data']['delete_group']['id']:
                return True
            else:
                return response
        except (TypeError, KeyError, IndexError):
            return response

    ####################################################################################################################
    def move_item_to_group(self, item_id, group_id):
        """
        Move an item from one group in a board to another.
        :param item_id: UUID of the item to move.
        :param group_id: UUID of the group to move the item to.
        :return: UUID of the moved item. Return None or error message on API call failure.
        """
        query_string = f'''mutation {{move_item_to_group (item_id: {item_id}, group_id: "{group_id}") {{id}}}}'''
        data = {'query': query_string}
        response = self.client.send_post_request(data)

        # Handle API response
        if not response:
            return None

        try:
            return response['data']['move_item_to_group']['id']
        except (TypeError, KeyError, IndexError):
            return response

    ####################################################################################################################
    # New - update_group
    def update_group(self, board_id, group_id, group_attribute, new_value):
        """
        Update an attribute of an existing group.
        :param board_id: ID of the board.
        :param group_id: ID of the group to update.
        :param group_attribute: Attribute to update ('title', 'color', 'position').
        :param new_value: New value for the attribute.
        :return: UUID of the updated group. Return None or error message on API call failure.
        """
        mutation = f'''mutation {{update_group (board_id: {board_id}, group_id: "{group_id}", group_attribute: 
        {group_attribute}, new_value: "{new_value}") {{id}}}}'''
        payload = {'query': mutation}
        response = self.client.send_post_request(payload)

        # Handle API response
        if not response:
            return None
        try:
            return response['data']['update_group']['id']
        except (TypeError, KeyError, IndexError):
            return response

    ####################################################################################################################
    # New - duplicate_group
    def duplicate_group(self, board_id, group_id, add_to_top=True):
        """
        Duplicate an existing group within a board.
        :param board_id: ID of the board.
        :param group_id: ID of the group to duplicate.
        :param add_to_top: Boolean to add the new group to the top of the board.
        :return: UUID of the duplicated group. Return None or error message on API call failure.
        """
        mutation = (f'mutation {{ duplicate_group (board_id: {board_id}, group_id: "{group_id}", add_to_top: '
                    f'{str(add_to_top).lower()}) {{ id }} }}')
        payload = {'query': mutation}
        response = self.client.send_post_request(payload)

        # Handle API response
        if not response:
            return None
        try:
            return response['data']['duplicate_group']['id']
        except (TypeError, KeyError, IndexError):
            return response

    ####################################################################################################################
    # New - archive_group
    def archive_group(self, board_id, group_id):
        """
        Archive an existing group within a board.
        :param board_id: The ID of the board where the group exists.
        :param group_id: The ID of the group to be archived.
        :return: True if the group is successfully archived. Return None or error message if the request fails.
        """
        query = f'''
        mutation {{
            archive_group (board_id: {board_id}, group_id: "{group_id}") {{
                id
            }}
        }}
        '''
        payload = {'query': query}
        response = self.client.send_post_request(payload)

        # Handle API response
        if not response:
            return None
        try:
            if response['data']['archive_group']['id']:
                return True
            else:
                return response
        except (TypeError, KeyError, IndexError):
            return response

    ####################################################################################################################
    # New - is_group_empty()
    def is_group_empty(self, board_id, group_id):
        """
        Check if a group in a board is empty (has zero items).
        :param board_id: The ID of the board.
        :param group_id: The ID of the group.
        :return: True if the group is empty, False otherwise. None or error message if the request fails.
        """
        query = (f'query {{ boards (ids: {board_id}) {{ groups (ids: "{group_id}") {{ items_page (limit: 1) {{ '
                 f'items {{ id }} }} }} }} }}')

        payload = {'query': query}
        response = self.client.send_post_request(payload)

        # Handle API response
        if not response:
            return None
        try:
            items = response["data"]["boards"][0]["groups"][0]["items_page"]["items"]
            return len(items) == 0  # True if no items, False otherwise
        except (TypeError, KeyError, IndexError):
            return response

    ####################################################################################################################





# pymonday/methods/tags.py

########################################################################################################################
# TAGS CLASS
########################################################################################################################
class Tags:
    def __init__(self, client):
        self.client = client

    ####################################################################################################################
    # New - create_or_get_tag
    def create_or_get_tag(self, tag_name, board_id=None):
        """
        Create a new tag or get an existing tag by name. Optionally associate the tag with a specific board.
        :param tag_name: The name of the tag to be created or retrieved.
        :param board_id: Optional. The ID of the board to associate the tag with.
        :return: The UUID of the created or retrieved tag. Return None or error message on API call failure.
        """
        mutation = f'''mutation {{create_or_get_tag (tag_name: "{tag_name}"{f', board_id: {board_id}' 
        if board_id else ''}) {{id name}}}}'''
        payload = {'query': mutation}

        response = self.client.send_post_request(payload)

        # Handle API response
        if not response:
            return None
        try:
            return response['data']['create_or_get_tag']['id']
        except (TypeError, KeyError, IndexError):
            return response

    ####################################################################################################################


# pymonday/methods/subitems.py

########################################################################################################################
# SUBITEMS CLASS
########################################################################################################################
class Subitems:
    def __init__(self, client):
        self.client = client

    ####################################################################################################################
    def get_subitems(self, item_id):
        """
        Get the UUIDs of the subitems of an item
        :param item_id: UUID of the parent Item
        :return: Array of subitem IDs. Return None or error message on API call failure.
        """
        query_string = f'''{{items (ids: {item_id}) {{subitems {{id}}}}}}'''
        data = {"query": query_string}
        response = self.client.send_post_request(data)
        if not response:
            return None
        try:
            subitem_ids = [item['id'] for item in response['data']['items'][0]['subitems']]
            return subitem_ids
        except (TypeError, KeyError, IndexError):
            return response

    ####################################################################################################################
    def get_subitem_names(self, item_id):
        """
        Get the UUIDs of the subitem IDs and names of an item
        :param item_id: UUID of the parent Item
        :return: Dict of subitem IDs & Names. Return None or error message on API call failure.
        """
        query_string = f'''{{items (ids: {item_id}) {{subitems {{id name}}}}}}'''
        data = {"query": query_string}
        response = self.client.send_post_request(data)
        if not response:
            return None
        try:
            subitem_ids = {item['id']: item['name'] for item in response['data']['items'][0]['subitems']}
            return subitem_ids
        except (TypeError, KeyError, IndexError):
            return response

    ####################################################################################################################
    def get_subitem_info(self, sub_item_id):
        """
        Get the Board ID and Parent ID from a Subitem.
        :param sub_item_id: UUID of the subitem.
        :return: Dictionary containing the parent ID and Board ID of the subitem. Return None or error message on API
        call failure.
        """
        query_string = f'''{{items(ids: {sub_item_id}) {{parent_item {{id}} board {{id}}}}}}'''
        data = {"query": query_string}
        response = self.client.send_post_request(data)
        if not response:
            return None
        try:
            parent_item = response['data']['items'][0]['parent_item']['id']
            board = response['data']['items'][0]['board']['id']
            subitem_dictionary = {"Parent Item": parent_item, "Subitem Board": board}
            return subitem_dictionary
        except (TypeError, KeyError, IndexError):
            return response

    ####################################################################################################################
    def create_subitem(self, parent_id, item_name, column_dictionary):
        """
        Create a new Subitem under a Parent Item.
        :param parent_id: UUID of the item to create the subitem under.
        :param item_name: Name of the new Subitem.
        :param column_dictionary: Dictionary containing column IDs, column types and values.
        :return: Dictionary containing the newly created Subitem ID and the board ID on which the subitem was created.
        Return None or error message on API call failure.
        """
        column_string = self.client.column_id_formatter(column_dictionary)
        query_string = f'''mutation {{create_subitem (parent_item_id: {parent_id}, item_name: "{item_name}", 
        column_values: "{column_string}") {{id board {{id}}}}}}'''
        data = {"query": query_string}
        response = self.client.send_post_request(data)
        if not response:
            return None
        try:
            board_id = response['data']['create_subitem']['board']['id']
            new_item_id = response['data']['create_subitem']['id']
            new_item_dict = {"Item ID": new_item_id, "Subitem Board": board_id}
            return new_item_dict
        except (TypeError, KeyError, IndexError):
            return response

    ####################################################################################################################


# pymonday/methods/activity_logs.py
from datetime import datetime

########################################################################################################################
# ACTIVITY LOGS CLASS
########################################################################################################################
class ActivityLogs:
    def __init__(self, client):
        self.client = client

    ####################################################################################################################
    # New - get_activity_logs
    def get_activity_logs(self, board_id, user_ids=None, limit=25, page=1, from_date=None, to_date=None,
                          column_ids=None, group_ids=None, item_ids=None):
        """
        Retrieve activity logs for a specific board.
        Required Scope: `activity:read`
        :param board_id: ID of the board to retrieve activity logs for.
        :param user_ids: List of user IDs to filter the logs (optional).
        :param limit: Number of activity logs to retrieve (default: 25).
        :param page: Page number to retrieve, starts from 1 (default: 1).
        :param from_date: Start date in 'dd.mm.yyyy' format (optional).
        :param to_date: End date in 'dd.mm.yyyy' format (optional).
        :param column_ids: List of column IDs to filter logs by (optional).
        :param group_ids: List of group IDs to filter logs by (optional).
        :param item_ids: List of item IDs to filter logs by (optional).
        :return: List of activity logs. None or error message if the request fails.
        """

        def convert_to_iso(date_str):
            try:
                return datetime.strptime(date_str, "%d.%m.%Y").isoformat() + "Z"
            except ValueError:
                print(f"Invalid date format: {date_str}. Expected format: dd.mm.yyyy")
                return None

        filters = []
        if user_ids:
            user_ids_string = ", ".join(map(str, user_ids))
            filters.append(f"user_ids: [{user_ids_string}]")
        if from_date:
            iso_from_date = convert_to_iso(from_date)
            if iso_from_date:
                filters.append(f'from: "{iso_from_date}"')
        if to_date:
            iso_to_date = convert_to_iso(to_date)
            if iso_to_date:
                filters.append(f'to: "{iso_to_date}"')
        if column_ids:
            column_ids_string = ", ".join(f'"{col_id}"' for col_id in column_ids)
            filters.append(f'column_ids: [{column_ids_string}]')
        if group_ids:
            group_ids_string = ", ".join(f'"{group_id}"' for group_id in group_ids)
            filters.append(f'group_ids: [{group_ids_string}]')
        if item_ids:
            item_ids_string = ", ".join(map(str, item_ids))
            filters.append(f'item_ids: [{item_ids_string}]')

        filters_string = ", ".join(filters)

        query = f'''query {{boards(ids: [{board_id}]) {{activity_logs(limit: {limit}, page: {page}{", " + filters_string 
        if filters_string else ""}) {{id event data account_id user_id created_at entity}}}}}}'''

        payload = {"query": query}
        response = self.client.send_post_request(payload)

        if not response:
            return None
        try:
            return response["data"]["boards"][0]["activity_logs"]
        except (TypeError, KeyError, IndexError):
            return response

    ####################################################################################################################


# pymonday/methods/columns.py

########################################################################################################################
# COLUMNS CLASS
########################################################################################################################
class Columns:
    def __init__(self, client):
        self.client = client

    ####################################################################################################################
    def change_column_values(self, board_id, item_id, column_dict):
        """
        Change the column values of an item. Multiple column values can be changed in a single query.
        :param board_id: UUID of the board the item is on.
        :param item_id: UUID of the Item the column values should be changed on.
        :param column_dict: Dictionary containing column IDs, column types and values.
        :return: UUID of the updated item. Return None or error message on API call failure.
        """
        column_string = self.client.column_id_formatter(column_dict)
        query_string = f'''mutation {{change_multiple_column_values (item_id: {item_id}, board_id: {board_id}, 
        column_values: "{column_string}") {{id}}}}'''
        data = {'query': query_string}
        response = self.client.send_post_request(data)
        if not response:
            return None
        try:
            return response['data']['change_multiple_column_values']['id']
        except (TypeError, KeyError, IndexError):
            return response

    ####################################################################################################################
    # New - create_column
    def create_column(self, board_id, title, column_type):
        """
        Create a new column in a board.
        :param board_id: The ID of the board where the column will be created.
        :param title: The title of the column to be created.
        :param column_type: The type of the column (e.g., text, date).
        :return: ID of the newly created column. None or error message if the request fails.
        """
        query = f'''mutation {{create_column (board_id: {board_id}, title: "{title}", column_type: {column_type}) {{id 
        title type }}}}'''
        payload = {'query': query}
        response = self.client.send_post_request(payload)

        # Handle API response
        if not response:
            return None
        try:
            return response["data"]["create_column"]["id"]
        except (TypeError, KeyError, IndexError):
            return response


    ####################################################################################################################
    # New - change_column_title
    def change_column_title(self, board_id, column_id, new_title):
        """
        Change the title of a column on a board.
        Required Scope: `boards:write`
        :param board_id: The ID of the board containing the column.
        :param column_id: The ID of the column to be updated.
        :param new_title: The new title to assign to the column.
        :return: UUID of the updated column. None or error message if the request fails.
        """
        mutation = f'''mutation {{change_column_title (board_id: {board_id}, column_id: "{column_id}", title: 
        "{new_title}") {{id title}}}}'''
        payload = {"query": mutation}
        response = self.client.send_post_request(payload)

        # Handle API response
        if not response:
            return None
        try:
            return response["data"]["change_column_title"]["id"]
        except (TypeError, KeyError, IndexError):
            return response

    ####################################################################################################################
    # New - get_column_metadata
    def get_column_metadata(self, board_id, column_ids):
        """
        Retrieve metadata for specific columns on a given board.
        Required Scope: `boards:read`
        :param board_id: ID of the board to retrieve column metadata from (required).
        :param column_ids: List of column IDs to retrieve (required).
        :return: Column metadata as a list of dictionaries. None or error message if the request fails.
        """
        # Build the query arguments
        arguments = []
        if column_ids:
            ids_str = ", ".join(f'"{col_id}"' for col_id in column_ids)
            arguments.append(f'ids: [{ids_str}]')

        # Combine arguments into the query string
        arguments_str = f"({', '.join(arguments)})" if arguments else ""

        # Construct the query
        query = f'''query {{boards (ids: {board_id}) {{columns {arguments_str} {{id title description type
        settings_str archived width }}}}}}'''
        payload = {"query": query}
        response = self.client.send_post_request(payload)

        # Handle API response
        if not response:
            return None
        try:
            return response["data"]["boards"][0]["columns"]
        except (TypeError, KeyError, IndexError):
            return response

    ####################################################################################################################
    def clear_column(self, board_id, item_id, column_id):
        """
        Clears the value of a given column in an item based on its column type.
        :param item_id: The ID of the item.
        :param board_id: The ID of the board.
        :param column_id: The ID of the column to clear.
        :return: UUID of the updated item. None or error message if the request fails.
        """
        # Retrieve column metadata to determine the column type
        metadata_response = self.get_column_metadata(board_id, [column_id])

        if not metadata_response or not metadata_response[0].get("type"):
            print(f"Failed to retrieve column metadata for column_id {column_id}")
            return None

        column_type = metadata_response[0]["type"]

        # Define clearing methods
        simple_null_columns = {
            "checkbox", "board_relation", "country", "dependency", "dropdown", "email", "hour",
            "link", "location", "long_text", "numbers", "people", "phone", "rating", "status",
            "tags", "text", "timeline", "week", "world_clock"
        }

        file_based_columns = {"file", "monday_doc"}

        if column_type in simple_null_columns:
            mutation = f'''mutation {{change_multiple_column_values (item_id: {item_id}, board_id: {board_id}, 
            column_values: "{{\\"{column_id}\\" : null}}") {{id}}}}'''
        elif column_type in file_based_columns:
            mutation = f'''mutation {{change_column_value (board_id: {board_id}, item_id: {item_id},
            column_id: "{column_id}", value: "{{\\"clear_all\\": true}}") {{id}}}}'''
        elif column_type == "date":
            mutation = f'''mutation {{change_simple_column_value (item_id: {item_id}, board_id: {board_id},
            column_id: "{column_id}", value: "") {{id}}}}'''
        else:
            return None

        payload = {'query': mutation}
        response = self.client.send_post_request(payload)

        # Handle API response
        if not response:
            return None
        try:
            return response["data"]["change_multiple_column_values"]["id"]
        except (TypeError, KeyError, IndexError):
            return response

    ####################################################################################################################



# pymonday/methods/boards.py

########################################################################################################################
# BOARDS CLASS
########################################################################################################################
class Boards:
    def __init__(self, client):
        self.client = client

    ####################################################################################################################
    def get_all_boards(self):
        """
        Get all boards from the platform.
        :return: Dictionary containing Board IDs and Board Names. Return None or error message on API call failure.
        """
        query_string = f'''
            query {{boards(limit: 1000) {{name id}}}}'''
        data = {'query': query_string}
        response = self.client.send_post_request(data)

        # Handle API response
        if not response:
            return None
        try:
            board_data = response['data']['boards']
            return {item['id']: item['name'] for item in board_data}
        except (TypeError, KeyError, IndexError):
            return response

    ####################################################################################################################
    def get_board_info(self, board_id):
        """
        Get the following information from a board: Name, ID, State, Permissions, Type, Workspace.
        :param board_id: UUID of the board.
        :return: Dictionary containing above values. Return None or error message on API call failure.
        """
        query_string = f'''
            query {{boards (ids: {board_id}) {{name state id permissions board_kind workspace_id}}}}'''
        data = {'query': query_string}
        response = self.client.send_post_request(data)

        # Handle API response
        if not response:
            return None
        try:
            return response['data']['boards'][0]
        except (TypeError, KeyError, IndexError):
            return response

    ####################################################################################################################
    def get_board_groups(self, board_id):
        """
        Get all groups of a specific board.
        :param board_id: UUID of the board.
        :return: Dictionary containing Group IDs and Group Names. Return None or error message on API call failure.
        """
        query_string = f'''
            query {{boards (ids: {board_id}) {{groups {{id title}}}}}}'''
        data = {'query': query_string}
        response = self.client.send_post_request(data)

        # Handle API response
        if not response:
            return None
        try:
            group_data = response['data']['boards'][0]['groups']
            return {item['title']: item['id'] for item in group_data}
        except (TypeError, KeyError, IndexError):
            return response

    ####################################################################################################################
    def get_board_from_item(self, item_id):
        """
        Get UUID of the board an item is on.
        :param item_id: UUID of the item.
        :return: Dictionary containing Board IDs and Board Names. Return None or error message on API call failure.
        """
        query_string = f'''
            query {{items(ids: {item_id}){{board{{id name}}}}}}'''
        data = {'query': query_string}
        response = self.client.send_post_request(data)

        # Handle API response
        if not response:
            return None
        try:
            return response['data']['items'][0]['board']
        except (TypeError, KeyError, IndexError):
            return response

    ####################################################################################################################
    def create_board(self, board_name, board_type, workspace_id):
        """
        Create a new board within a specific workspace.
        :param board_name: Required. The Name of the new board
        :param board_type: Required. Board Visibility. Options: public, private, share
        :param workspace_id: Required. UUID of the workspace to create the board in
        :return: UUID of newly created board. Return None or error message on API call failure.
        """
        query_string = f'''mutation 
            {{create_board (board_name: "{board_name}", board_kind: {board_type}, workspace_id: {workspace_id}) {{id}}}}'''
        data = {'query': query_string}
        response = self.client.send_post_request(data)

        # Handle API response
        if not response:
            return None
        try:
            return response['data']['create_board']['id']
        except (TypeError, KeyError, IndexError):
            return response

    ####################################################################################################################
    def delete_board(self, board_id):
        """
        Delete a board.
        :param board_id: UUID of the board to delete.
        :return: True if successful. Return None or error message on API call failure.
        """
        query_string = f'''mutation 
                    {{delete_board (board_id: {board_id}) {{id}}}}'''
        data = {'query': query_string}
        response = self.client.send_post_request(data)

        # Handle API response
        if not response:
            return None

        try:
            if response['data']['delete_board']['id']:
                return True
        except (TypeError, KeyError, IndexError):
            return response


    ####################################################################################################################
    def update_board_description(self, board_id, description):
        """
        Update the description of a specific board.
        :param board_id: UUID of the board to update.
        :param description: Text String to be used as the board description.
        :return: Response from the API. Return None or error message on API call failure.
        """
        query_string = f'''mutation {{
            update_board(board_id: {board_id}, board_attribute: description, new_value: "{description}")}}'''
        data = {'query': query_string}
        response = self.client.send_post_request(data)

        # Handle API response
        if not response:
            return None
        try:
            return response['data']['update_board']
        except (TypeError, KeyError, IndexError):
            return None


    ####################################################################################################################
    def archive_board(self, board_id):
        """
        Archive a board.
        :param board_id: UUID of the board to archive.
        :return: True if successful. Return None or error message on API call failure.
        """
        query_string = f'''mutation {{archive_board (board_id: {board_id}) {{id}}}}'''
        data = {'query': query_string}
        response = self.client.send_post_request(data)

        # Handle API response
        if not response:
            return None
        try:
            if response['data']['archive_board']['id']:
                return True
            else:
                return response
        except (TypeError, KeyError, IndexError):
            return response


    ####################################################################################################################
    # New - duplicate_board
    def duplicate_board(self, board_id, duplicate_type='duplicate_board_with_structure', board_name=None,
                        keep_subscribers=False, workspace_id=None, folder_id=None):
        """
        Duplicate a board with its items and groups.
        :param board_id: The unique identifier of the board to duplicate.
        :param duplicate_type: The duplication type. Options are:
                               'duplicate_board_with_structure',
                               'duplicate_board_with_pulses',
                               'duplicate_board_with_pulses_and_updates'.
                               Default is 'duplicate_board_with_structure'.
        :param board_name: Optional new name for the duplicated board. If omitted, it will be automatically generated.
        :param keep_subscribers: Boolean indicating whether to duplicate the subscribers to the new board. Defaults to False.
        :param workspace_id: Optional ID of the destination workspace. If omitted, it will default to the original board's workspace.
        :param folder_id: Optional ID of the destination folder within the destination workspace. Required if duplicating to another workspace; otherwise, optional.
        :return: New board UUID. Return None or error message on API call failure.
        """
        mutation = f'''mutation {{duplicate_board (board_id: {board_id}, duplicate_type: {duplicate_type.lower()}
        {f', board_name: "{board_name}"' if board_name else ''} {f', keep_subscribers: {str(keep_subscribers).lower()}'}
        {f', workspace_id: {workspace_id}' if workspace_id else ''} {f', folder_id: {folder_id}' if folder_id else ''}) 
        {{board {{id name}}}}}}'''
        payload = {'query': mutation}
        response = self.client.send_post_request(payload)

        # Handle API response
        if not response:
            return None
        try:
            return response['data']['duplicate_board']['board']['id']
        except (TypeError, KeyError, IndexError):
            return response

    ####################################################################################################################
    # New - get_board_views
    def get_board_views(self, board_id):
        """
        Retrieve the views of a specific board.
        Required Scope: `boards:read`
        :param board_id: ID of the board whose views are to be retrieved.
        :return: List of board views or None if the request fails. Return None or error message on API call failure.
        """
        query = f'''query {{boards(ids: {board_id}) {{views {{id name type settings_str view_specific_data_str}}}}}}'''
        payload = {"query": query}
        response = self.client.send_post_request(payload)

        # Handle API response
        if not response:
            return None

        try:
            return response["data"]["boards"][0]["views"]
        except (TypeError, KeyError, IndexError):
            return response

    ####################################################################################################################


# pymonday/methods/account.py

########################################################################################################################
# ACCOUNT CLASS
########################################################################################################################
class Account:
    def __init__(self, client):
        self.client = client

    ####################################################################################################################
    # New - get_account_details
    def get_account_details(self):
        """
        Retrieve the details of the account.
        Required Scope: `account:read`

        :return: Dictionary containing the account details. None or error message if the request fails.
        """
        query = '''query {users {account {id name logo active_members_count country_code first_day_of_the_week 
        show_timeline_weekends tier slug plan {period} products {id kind} sign_up_product_kind}}}'''
        payload = {"query": query}
        response = self.client.send_post_request(payload)  # Use the existing client's session

        if not response:
            return None
        try:
            return response["data"]["users"][0]["account"]
        except (TypeError, KeyError, IndexError):
            return response

    ####################################################################################################################




# pymonday/methods/workspaces.py

########################################################################################################################
# WORKSPACES CLASS
########################################################################################################################
class Workspaces:
    def __init__(self, client):
        self.client = client

    ####################################################################################################################
    def get_workspaces(self):
        """
        Get all monday.com workspaces
        :return: List of dictionaries containing workspaces IDs and names. Returns None or error message on API call failure.
        """
        query_string = f'''query {{workspaces {{id name kind description}}}}'''
        data = {'query': query_string}
        response = self.client.send_post_request(data)
        if not response:
            return None
        try:
            workspace_data = response['data']['workspaces']
            return workspace_data
        except (TypeError, KeyError, IndexError):
            return response

    ####################################################################################################################
    # New - create_workspace
    def create_workspace(self, name, kind='open', description=None):
        """
        Create a new workspace.
        :param name: The name of the new workspace.
        :param kind: The type of workspace ('open' or 'closed'). Default is 'open'.
        :param description: Optional description of the workspace.
        :return: UUID of the newly created workspace. Return None or error message on API call failure.
        """
        mutation = f'''mutation {{create_workspace (name: "{name}", kind: {kind.lower()}
                {f', description: "{description}"' if description else ''}) {{id name description}}}}'''
        payload = {'query': mutation}
        response = self.client.send_post_request(payload)
        if not response:
            return None
        try:
            return response['data']['create_workspace']['id']
        except (TypeError, KeyError, IndexError):
            return response

    ####################################################################################################################
    # New - add_users_to_workspace
    def add_users_to_workspace(self, workspace_id, user_ids, kind='subscriber'):
        """
        Add users to a workspace.
        :param workspace_id: The ID of the workspace.
        :param user_ids: List of user IDs to add to the workspace.
        :param kind: The role of the users in the workspace ('subscriber' or 'owner'). Default is 'subscriber'.
        :return: JSON response confirming the addition of users.
        """
        user_ids_str = ', '.join(map(str, user_ids))
        mutation = f'''mutation {{add_users_to_workspace (workspace_id: {workspace_id}, user_ids: [{user_ids_str}],
                kind: {kind.lower()}) {{id}}}}'''
        payload = {'query': mutation}
        response = self.client.send_post_request(payload)
        if not response:
            return None
        try:
            return response['data']['add_users_to_workspace']['id']
        except (TypeError, KeyError, IndexError):
            return response

    ####################################################################################################################
    # New - delete_workspace
    def delete_workspace(self, workspace_id):
        """
        Delete a workspace.
        :param workspace_id: The ID of the workspace to delete.
        :return: True if successful. Return None or error message on API call failure.
        """
        mutation = f'''mutation {{delete_workspace (workspace_id: {workspace_id}) {{id}}}}'''
        payload = {'query': mutation}
        response = self.client.send_post_request(payload)
        if not response:
            return None
        try:
            if response['data']['delete_workspace']['id']:
                return True
            else:
                return response
        except (TypeError, KeyError, IndexError):
            return response

    ####################################################################################################################
    # New - delete_users_from_workspace
    def delete_users_from_workspace(self, workspace_id, user_ids):
        """
        Remove users from a workspace.
        :param workspace_id: The ID of the workspace.
        :param user_ids: List of user IDs to remove from the workspace.
        :return: True if successful. Return None or error message on API call failure.
        """
        user_ids_str = ', '.join(map(str, user_ids))
        mutation = f'''mutation {{delete_users_from_workspace (workspace_id: {workspace_id}, user_ids: [{user_ids_str}]
            ) {{id}}}}'''
        payload = {'query': mutation}
        response = self.client.send_post_request(payload)
        if not response:
            return None
        try:
            if response['data']['delete_users_from_workspace']['id']:
                return True
            else:
                return response
        except (TypeError, KeyError, IndexError):
            return response

    ####################################################################################################################
    # New - add_teams_to_workspace
    def add_teams_to_workspace(self, workspace_id, team_ids, kind="subscriber"):
        """
        Add teams to a workspace.
        :param workspace_id: The ID of the workspace.
        :param team_ids: List of team IDs to add to the workspace.
        :param kind: The subscriber's role ('owner' or 'subscriber'). Default is 'subscriber'.
        :return: True if successful. Return None or error message on API call failure.
        """
        # Convert team_ids list to GraphQL array format
        team_ids_str = ', '.join(map(str, team_ids))

        mutation = f'''mutation {{add_teams_to_workspace (workspace_id: {workspace_id}, team_ids: [{team_ids_str}],
                kind: {kind.lower()}) {{id}}}}'''
        payload = {'query': mutation}
        response = self.client.send_post_request(payload)
        if not response:
            return None
        try:
            if response['data']['add_teams_to_workspace']['id']:
                return True
            else:
                return response
        except (TypeError, KeyError, IndexError):
            return response

    ####################################################################################################################
    # New - delete_teams_from_workspace
    def delete_teams_from_workspace(self, workspace_id, team_ids):
        """
        Remove teams from a workspace.
        :param workspace_id: The ID of the workspace.
        :param team_ids: List of team IDs to remove from the workspace.
        :return: True if successful. Return None or error message on API call failure.
        """
        # Convert team_ids list to GraphQL array format
        team_ids_str = ', '.join(map(str, team_ids))

        mutation = f'''mutation {{delete_teams_from_workspace (workspace_id: {workspace_id}, team_ids: [{team_ids_str}]
            ) {{id}}}}'''
        payload = {'query': mutation}
        response = self.client.send_post_request(payload)
        if not response:
            return None
        try:
            if response['data']['delete_teams_from_workspace']['id']:
                return True
            else:
                return response
        except (TypeError, KeyError, IndexError):
            return response

    ####################################################################################################################
    # New - update_workspace
    def update_workspace(self, workspace_id, name=None, description=None):
        """
        Update the attributes of a workspace.
        :param workspace_id: The ID of the workspace.
        :param name: The new name for the workspace (optional).
        :param description: The new description for the workspace (optional).
        :return: The updated workspace data. Return None or error message on API call failure.
        """
        # Build the attributes object dynamically based on provided arguments
        attributes = []
        if name:
            attributes.append(f'name: "{name}"')
        if description:
            attributes.append(f'description: "{description}"')
        attributes_str = ', '.join(attributes)

        mutation = f'''mutation {{update_workspace (id: {workspace_id}, attributes: {{ {attributes_str} }}
            ) {{id name description}}}}'''
        payload = {'query': mutation}
        response = self.client.send_post_request(payload)
        if not response:
            return None
        try:
            return response['data']['update_workspace']
        except (TypeError, KeyError, IndexError):
            return response

    ####################################################################################################################


# pymonday/methods/users.py

########################################################################################################################
# USERS CLASS
########################################################################################################################
class Users:
    def __init__(self, client):
        self.client = client

    ####################################################################################################################
    def get_all_users(self):
        """
        Get the names of all platform users and their UUIDs
        :return: Dictionary of item IDs and usernames. Return None or error message on API call failure.
        """
        query_string = f'''query {{users(limit: 200) {{id name email phone country_code created_at last_activity}}}}'''
        data = {'query': query_string}
        response = self.client.send_post_request(data)
        if not response:
            return None
        try:
            user_data = response['data']['users']
            return {item['id']: item['name'] for item in user_data}
        except (TypeError, KeyError, IndexError):
            return response

    ####################################################################################################################
    def get_user_info(self, user_id):
        """
        Get all details of a specific user. One UUID permitted per query.
        :param user_id: UUID of the User.
        :return: Dictionary containing user specific information. Return None or error message on API call failure.
        """
        query_string = f'''query {{
        users(ids: {user_id}) {{id name email phone country_code created_at last_activity}}}}'''
        data = {'query': query_string}
        response = self.client.send_post_request(data)
        if not response:
            return None
        try:
            user_data = response['data']['users'][0]
            return user_data
        except (TypeError, KeyError, IndexError):
            return response

    ####################################################################################################################
    def add_user_to_board(self, board_id, user_ids):
        """
        Add users to a board as subscribers.
        :param board_id: UUID of the board. Single Integer value.
        :param user_ids: UUIDs of the users to add. Should be a list of integers.
        :return: List containing the IDs of the newly added users. Return None or error message on API call failure.
        """
        query_string = f'''mutation {{
        add_users_to_board (board_id: {board_id}, user_ids: {user_ids}, kind: subscriber) {{id}}}}'''
        data = {'query': query_string}
        response = self.client.send_post_request(data)
        if not response:
            return None
        try:
            return response['data']['add_users_to_board']
        except (TypeError, KeyError, IndexError):
            return response

    ####################################################################################################################
    def remove_user_from_board(self, board_id, user_ids):
        """
        Remove users from a board as subscribers.
        :param board_id: UUID of the board. Single Integer value.
        :param user_ids: UUIDs of the users to remove. Should be a list of integers.
        :return: True if successful. Return None or error message on API call failure.
        """
        query_string = f'''mutation {{
            delete_subscribers_from_board (board_id: {board_id}, user_ids: {user_ids}) {{id}}}}'''
        data = {'query': query_string}
        response = self.client.send_post_request(data)
        if not response:
            return None
        try:
            if response['data']['delete_subscribers_from_board']:
                return True
            else:
                return response
        except (TypeError, KeyError, IndexError):
            return response

    ####################################################################################################################


# pymonday/methods/docs.py

########################################################################################################################
# DOCS CLASS
########################################################################################################################
class Docs:
    def __init__(self, client):
        self.client = client

    ####################################################################################################################
    # New - fetch_docs
    def fetch_docs(self, object_ids=None, limit=25, page=1):
        """
        Fetch metadata for docs with pagination.
        :param object_ids: Specific doc IDs to return (optional).
        :param limit: Limit on the number of docs to retrieve (default: 25, max: 100).
        :param page: Page number to fetch (default: 1).
        :return: List of dictionaries containing doc details. None or error message if the request fails.
        """
        object_ids_string = f"object_ids: {object_ids}" if object_ids else ""
        query = f'''query {{docs ({object_ids_string}, limit: {limit}, page: {page}) {{id object_id settings created_by 
        {{id name}}}}}}'''
        payload = {'query': query}
        response = self.client.send_post_request(payload)
        if not response:
            return None
        try:
            return response['data']['docs']
        except (TypeError, KeyError, IndexError):
            return response

    ####################################################################################################################
    # New - create_doc
    def create_doc(self, workspace_id, name, kind="private"):
        """
        Create a new doc in a workspace.
        :param workspace_id: ID of the workspace.
        :param name: Name of the new doc.
        :param kind: Doc kind ('private', 'public', or 'share').
        :return: UUID of the newly created doc. Return None or error message on API call failure.
        """
        # Validate and format `kind`
        kind = kind.lower()
        if kind not in {"private", "public", "share"}:
            raise ValueError("Invalid 'kind'. Must be 'private', 'public', or 'share'.")

        # Construct the GraphQL mutation
        mutation = (f'mutation {{ create_doc (location: {{ workspace: {{ workspace_id: {workspace_id}, name: "{name}", '
                    f'kind: {kind} }} }} ) {{ id }} }}')
        payload = {'query': mutation}
        response = self.client.send_post_request(payload)
        if not response:
            return None
        try:
            return response['data']['create_doc']['id']
        except (TypeError, KeyError, IndexError):
            return response

    ####################################################################################################################


# pymonday/methods/files.py

########################################################################################################################
# IMPORTS
########################################################################################################################
import os


########################################################################################################################
# FILES CLASS
########################################################################################################################
class Files:
    def __init__(self, client):
        self.client = client

    ####################################################################################################################
    def get_assets(self, item_id):
        """
        Get the assets(files) associated with an item.
        :param item_id: UUID of the item.
        :return: Array of dictionaries containing asset data. Return None or full error message if request fails.
        """
        query_string = f'''{{items(ids:{item_id}){{assets {{id name file_size created_at public_url url}}}}}}'''
        data = {'query': query_string}
        response = self.client.send_post_request(data)
        if not response:
            return None
        try:
            return response['data']['items'][0]['assets']
        except (TypeError, KeyError, IndexError):
            return response

    ####################################################################################################################
    def upload_file_to_column(self, item_id, column_id, filepath):
        """
        Upload a local file to a file type column of an item. Remote files not supported.
        :param item_id: UUID of the item
        :param column_id: Column ID to upload the file to.
        :param filepath: Absolute Path to file on the local system. File Extension required.
        :return: UUID of the asset. Return None or full error message if request fails.
        """
        file_name = os.path.basename(filepath)
        payload = {
            'query': f'mutation add_file($file: File!) {{add_file_to_column (item_id: {item_id}, '
                     f'column_id: "{column_id}" file: $file) {{id}}}}', 'map': '{"column_file": "variables.file"}'}
        files = [('column_file', (f'{file_name}', open(filepath, 'rb'), 'application/octet-stream'))]

        response = self.client.upload_file(payload, files)
        if not response:
            return None
        try:
            return response['data']['add_file_to_column']['id']
        except (TypeError, KeyError, IndexError):
            return response

    ####################################################################################################################
    def add_file_to_update(self, update_id, file_path):
        """
        Upload a local file to an update. Remote files not supported.
        :param update_id: UUID of the update.
        :param file_path: Absolute Path to the local file. File Extension Required.
        :return: UUID of the asset. Return None or full error message if request fails.
        """
        file_name = os.path.basename(file_path)
        payload = {
            'query': f'mutation ($file: File!) {{add_file_to_update(file: $file, update_id: {update_id}) {{id}}}}',
            'map': '{"update_file":"variables.file"}'}
        files = [('update_file', (f'{file_name}', open(f'{file_path}', 'rb'), 'application/octet-stream'))]
        response = self.client.upload_file(payload, files)
        if not response:
            return None
        try:
            return response['data']['add_file_to_update']['id']
        except (TypeError, KeyError, IndexError):
            return response

        ####################################################################################################################


# pymonday/monday_api_client.py

########################################################################################################################
# IMPORTS
########################################################################################################################
import httpx
import asyncio
import time
from .config import monday_api_URL, monday_file_URL, MAX_RETRIES
from .columns import column_formats

# Import all method groups
from .methods.account import Account
from .methods.activity_logs import ActivityLogs
from .methods.boards import Boards
from .methods.columns import Columns
from .methods.docs import Docs
from .methods.doc_blocks import DocBlocks
from .methods.files import Files
from .methods.folders import Folders
from .methods.groups import Groups
from .methods.items import Items
from .methods.notifications import Notifications
from .methods.subitems import Subitems
from .methods.tags import Tags
from .methods.teams import Teams
from .methods.updates import Updates
from .methods.users import Users
from .methods.workspaces import Workspaces

########################################################################################################################
# MONDAY API CLIENT
########################################################################################################################
class MondayAPIClient:
    """
    Client for interacting with the Monday.com GraphQL API.
    """

    def __init__(self, api_key):
        self.access_token = api_key
        self.session = httpx.Client()
        self.column_values = column_formats
        self.results = []
        self.session.headers = {
            "Authorization": f"Bearer {self.access_token}",
            "Content-Type": "application/json",
            "API-Version": "2025-01"
        }

        # Initialize modules, passing self so they share the same session
        self.account = Account(self)
        self.activity_logs = ActivityLogs(self)
        self.boards = Boards(self)
        self.columns = Columns(self)
        self.docs = Docs(self)
        self.doc_blocks = DocBlocks(self)
        self.files = Files(self)
        self.folders = Folders(self)
        self.groups = Groups(self)
        self.items = Items(self)
        self.notifications = Notifications(self)
        self.subitems = Subitems(self)
        self.tags = Tags(self)
        self.teams = Teams(self)
        self.updates = Updates(self)
        self.users = Users(self)
        self.workspaces = Workspaces(self)

    ####################################################################################################################
    # MONDAY API CLIENT CLASS METHODS
    ####################################################################################################################
    def send_post_request(self, payload):
        """
        API POST Request using HTTPX Session
        :param payload: JSON query string
        :return: JSON formatted data from the HTTP response. Returns None on API call failure.
        """
        for _ in range(MAX_RETRIES):
            response = self.session.post(url=monday_api_URL, json=payload, timeout=30.0)
            if response.status_code == 200:
                return response.json()
            print(response.content)
            time.sleep(5)

    ####################################################################################################################
    def upload_file(self, payload, files):
        """
        API POST Request for file uploads.
        :param payload: JSON GraphQL query string.
        :param files: Files to be uploaded.
        :return: JSON response from API call.
        """
        file_headers = {'Authorization': self.access_token, "API-Version": "2024-07"}
        for _ in range(MAX_RETRIES):
            response = httpx.request(
                method="POST", url=monday_file_URL, data=payload, files=files, headers=file_headers)
            if response.status_code == 200:
                return response.json()
            print(response.content)
            time.sleep(5)

    ####################################################################################################################
    async def async_post(self, payload):
        """
        Asynchronous API call with concurrency limit handling.
        """
        async with httpx.AsyncClient(headers=self.session.headers) as client:
            for _ in range(MAX_RETRIES):
                response = await client.post(url=monday_api_URL, json=payload, timeout=60)
                if response.status_code == 200:
                    self.results.append(response.json())
                    return

                if response.status_code == 429 or "maxConcurrencyExceeded" in response.text:
                    error_data = response.json()
                    retry_seconds = error_data.get("errors", [{}])[0].get("extensions", {}).get("retry_in_seconds", 15)
                    print(f"Concurrency limit exceeded. Retrying in {retry_seconds} seconds...")
                    await asyncio.sleep(retry_seconds)
                else:
                    print(response.content)
                    await asyncio.sleep(5)

        print("Max retries exceeded.")
        return None

    ####################################################################################################################
    def get_next_page(self, cursor, items, item_attributes):
        """
        Get the next set of 100 records using cursor-based pagination.
        :param cursor: Token-based cursor for retrieving the next set of records.
        :param items: Initial items list.
        :param item_attributes: Attributes to retrieve.
        :return: List of item data.
        """
        while cursor is not None:
            subsequent_query = f'''query
            {{next_items_page (limit: 100, cursor: "{cursor}") {{cursor items {{{item_attributes}}}}}}}'''
            next_data = {'query': subsequent_query}
            next_response = self.send_post_request(next_data)

            if not next_response:
                return None
            next_items = [item for item in next_response['data']['next_items_page']['items']]
            items = items + next_items
            cursor = next_response['data']['next_items_page']['cursor']

        return items

    ####################################################################################################################
    async def column_task_handler(self, item_ids, columns):
        """
        Async task handler for column value retrieval.
        :param item_ids: UUID of the Item to retrieve column values for.
        :param columns: UUID of the columns values to retrieve.
        :return: Json formatted Data from the HTTP response. Return None on API call failure.
        """
        column_string = ""
        for column in columns:
            column_string = column_string + f"\"{column}\", "
        column_string = column_string[:-2]
        column_string = f"[{column_string}]"

        tasks = []
        for each_item in item_ids:
            query_string = f'''
            {{items(ids: {each_item}) {{id name parent_item{{id name}}column_values(ids: {column_string}) 
            {{id text value column{{id title}} 
            ... on MirrorValue{{display_value}} 
            ... on BoardRelationValue{{display_value linked_items{{id}}}}}}}}}}'''
            payload = {'query': query_string}
            tasks.append(self.async_post(payload))
        await asyncio.gather(*tasks)

    ####################################################################################################################
    def column_id_formatter(self, column_dictionary):
        """
        Takes the column values dictionary and converts it into a correctly formatted GraphQL query string.
        :param column_dictionary: Dictionary containing column id, column type, and required values for the query.
        :return: JSON formatted GraphQL query for column values.
        """
        query_string = "{"
        for (key, value) in column_dictionary.items():
            value_arguments = [key]
            for each_value in value['values']:
                value_arguments.append(each_value)
            column_string = self.column_values[value['type']].format(*value_arguments)
            query_string += f"{column_string}, "

        query_string = query_string.rstrip(", ")  # Remove the last comma
        query_string += "}"
        return query_string

    ####################################################################################################################


# pymonday/methods/teams.py

########################################################################################################################
# TEAMS CLASS
########################################################################################################################
class Teams:
    def __init__(self, client):
        self.client = client

    ####################################################################################################################
    def get_teams(self):
        """
        Get Names and IDs of all teams.
        :return: Dictionary of IDs and Names. Return None or error message on API call failure.
        """
        query_string = f'''{{teams {{name id}}}}'''
        data = {'query': query_string}
        response = self.client.send_post_request(data)
        if not response:
            return None
        try:
            team_data = response['data']['teams']
            teams = {item['id']: item['name'] for item in team_data}
            return teams
        except (TypeError, KeyError, IndexError):
            return response

    ####################################################################################################################
    def get_team_members(self, team_ids):
        """
        Get ID, Name & Email of team members.
        :param team_ids: UUIDs of the Teams to be retrieved. List of integers.
        :return: Dictionary of Teams and member data. Return None or error message on API call failure.
        """
        query_string = f'''{{teams (ids: {team_ids}){{name users {{name id email}}}}}}'''
        data = {'query': query_string}
        response = self.client.send_post_request(data)
        if not response:
            return None
        try:
            member_data = response['data']['teams']
            team_dictionary = {item['name']: item['users'] for item in member_data}
            return team_dictionary
        except (TypeError, KeyError, IndexError):
            return response

    ####################################################################################################################


# pymonday/methods/doc_blocks.py

########################################################################################################################
# IMPORTS
########################################################################################################################
import json


########################################################################################################################
# DOC BLOCKS CLASS
########################################################################################################################
class DocBlocks:
    def __init__(self, client):
        self.client = client

    ####################################################################################################################
    # New - fetch_doc_blocks
    def fetch_doc_blocks(self, doc_id, limit=50, page=1):
        """
        Fetch metadata for all blocks within a specific doc with pagination.
        :param doc_id: ID of the doc.
        :param limit: Number of blocks to retrieve per request (default: 50, max: 100).
        :param page: Page number to fetch (default: 1).
        :return: JSON response containing block details. Return None or error message on API call failure.
        """
        query = f'''query {{docs (ids: {doc_id}) {{blocks (limit: {limit}, page: {page}) {{id type content}}}}}}'''
        payload = {'query': query}

        # Send request and validate response
        response = self.client.send_post_request(payload)
        if not response:
            return None
        try:
            return response['data']['docs'][0]['blocks']
        except (TypeError, KeyError, IndexError):
            return response

    ####################################################################################################################
    # New - create_doc_block
    def create_doc_block(self, doc_id, block_type, content, after_block_id=None):
        """
        Create a new block within a specific doc.
        :param doc_id: ID of the doc (required).
        :param block_type: Type of the block (required, e.g., 'normal_text', 'table').
        :param content: JSON-formatted string or dictionary representing the block content (required)
        (for content field information check https://developer.monday.com/api-reference/reference/blocks).
        :param after_block_id: ID of the block that will be above the new block (optional).
        :return: Created block's ID. Return None or error message on API call failure.
        """
        # Ensure `content` is properly JSON-encoded
        formatted_content = json.dumps(content) if isinstance(content, dict) else content

        # Escape double quotes in the JSON content to ensure compatibility with GraphQL
        escaped_content = formatted_content.replace('"', '\\"')

        # Build the mutation as a single concatenated string
        mutation = (f'mutation {{ create_doc_block (type: {block_type}, doc_id: {doc_id}, ' +
                    (f'after_block_id: "{after_block_id}", ' if after_block_id else '') +
                    f'content: "{escaped_content}") {{ id }} }}')


        payload = {'query': mutation}
        response = self.client.send_post_request(payload)

        # Handle API response
        if not response:
            return None
        try:
            return response['data']['create_doc_block']['id']
        except (TypeError, KeyError, IndexError):
            return response

    ####################################################################################################################
    # New - update_doc_block
    def update_doc_block(self, block_id, content):
        """
        Update the content of an existing block.
        :param block_id: ID of the block to update (required).
        :param content: New content for the block. Must be a valid JSON object or string (required)
        (for content field information check https://developer.monday.com/api-reference/reference/blocks).
        :return: Updated block's ID. Return None or error message on API call failure.
        """
        # Ensure `content` is properly JSON-encoded
        formatted_content = json.dumps(content) if isinstance(content, dict) else content

        # Escape double quotes in the JSON content for GraphQL compatibility
        escaped_content = formatted_content.replace('"', '\\"')

        # Construct the mutation
        mutation = f'mutation {{ update_doc_block (block_id: "{block_id}", content: "{escaped_content}") {{ id }} }}'
        payload = {'query': mutation}
        response = self.client.send_post_request(payload)

        # Handle API response
        if not response:
            return None
        try:
            return response['data']['update_doc_block']['id']
        except (TypeError, KeyError, IndexError):
            return response

    ####################################################################################################################
    # New - delete_doc_block
    def delete_doc_block(self, block_id):
        """
        Delete a block from a doc.
        :param block_id: ID of the block to delete (required).
        :return: True if successful. Return None or error message on API call failure.
        """
        # Ensure the block_id is properly quoted for GraphQL
        mutation = f'mutation {{ delete_doc_block (block_id: "{block_id}") {{ id }} }}'
        payload = {'query': mutation}
        response = self.client.send_post_request(payload)

        if not response:
            return None
        try:
            if response['data']['delete_doc_block']['id']:
                return True
            else:
                return response
        except (TypeError, KeyError, IndexError):
            return response

    ####################################################################################################################


# pymonday/methods/items.py

########################################################################################################################
# IMPORTS
########################################################################################################################
import asyncio


########################################################################################################################
# ITEMS CLASS
########################################################################################################################
class Items:
    def __init__(self, client):
        self.client = client

    ####################################################################################################################
    def get_item_ids_from_group(self, board_id, group_id):
        """
        Get all item IDs from a specific group on a board. Cursor Based Pagination Required. Records limited to 100
        per call.
        :param board_id: UUID of the board.
        :param group_id: UUID of the group.
        :return: Array of Item IDs. Return None or error message on API call failure.
        """
        item_attributes = "id"
        initial_query = f'''query {{boards (ids: {board_id}) {{groups(ids: "{group_id}") {{items_page (limit: 500) {{
        cursor items {{{item_attributes}}}}}}}}}}}'''
        data = {'query': initial_query}
        response = self.client.send_post_request(data)

        if not response:
            return None

        try:
            items_page = response["data"]["boards"][0]["groups"][0]["items_page"]
            item_ids = [item['id'] for item in items_page["items"]]
            cursor = items_page["cursor"]

            if cursor is None:
                return item_ids

            all_item_ids = self.client.get_next_page(cursor, item_ids, item_attributes)
            return [int(item['id']) for item in all_item_ids]

        except (TypeError, KeyError, IndexError):
            return response

    ####################################################################################################################
    def get_items_page_from_group(self, board_id, group_id, columns):
        """
        Retrieve items from a specific group in a board.

        :param board_id: ID of the board.
        :param group_id: ID of the group.
        :param columns: List of column IDs to retrieve.
        :return: List of items. Return None or error message on API call failure.
        """
        column_string = ", ".join(f'"{col}"' for col in columns)
        column_string = f"[{column_string}]"

        item_attributes = f'''name id parent_item{{id name }} column_values(ids: {column_string}) {{text value 
        column{{id title}} ... on BoardRelationValue {{display_value linked_items {{id}}}} ... on MirrorValue 
        {{display_value}}}}'''

        query = f'''query {{boards(ids: [{board_id}]) {{groups(ids: "{group_id}") {{items_page(limit: 50) {{cursor
        items {{{item_attributes}}}}}}}}}}}'''
        payload = {'query': query}
        response = self.client.send_post_request(payload)

        # Handle API response
        if not response:
            return None
        try:
            items_data = response['data']['boards'][0]['groups'][0]['items_page']
            items = items_data.get('items', [])
            cursor = items_data.get('cursor')

            return items if cursor is None else self.client.get_next_page(cursor, items, item_attributes)
        except (TypeError, KeyError, IndexError):
            return response

    ####################################################################################################################
    def get_items_from_column(self, board_id, column_id, value, columns):
        """
        Retrieve items from a board filtered by a column value.

        :param board_id: ID of the board.
        :param column_id: ID of the column to filter.
        :param value: Value to filter items by.
        :param columns: List of column IDs to retrieve.
        :return: List of items. Return None or error message on API call failure.
        """
        column_string = ", ".join(f'"{col}"' for col in columns)
        column_string = f"[{column_string}]"

        item_attributes = f'''name id parent_item{{id name }} column_values(ids: {column_string}) {{text value column
        {{id title}} ... on BoardRelationValue {{display_value linked_items {{id}}}} ... on MirrorValue 
        {{display_value}}}}'''

        query = f'''query {{items_page_by_column_values (limit: 50, board_id: {board_id}, columns: [{{column_id: 
        "{column_id}", column_values: ["{value}"]}}]) {{cursor items {{{item_attributes}}}}}}}'''
        data = {'query': query}
        response = self.client.send_post_request(data)

        # Handle API response
        if not response:
            return None
        try:
            items_data = response['data']['items_page_by_column_values']
            items = items_data.get('items', [])
            cursor = items_data.get('cursor')
            return items if cursor is None else self.client.get_next_page(cursor, items, item_attributes)
        except (TypeError, KeyError, IndexError):
            return response

    ####################################################################################################################
    def get_items_with_status(self, board_id, group_id, columns, column_id, index_value):
        """
        Retrieve items from a specific group on a board where a column matches a given status/index value.
        :param board_id: UUID of the board.
        :param group_id: UUID of the group.
        :param columns: List of column IDs to retrieve values from.
        :param column_id: The column to filter items by.
        :param index_value: The value to match in the column.
        :return: List of items. Return None or error message on API call failure.
        """
        column_string = ", ".join(f"\"{column}\"" for column in columns)
        item_attributes = f'''name id parent_item{{id name}} column_values(ids: [{column_string}]) {{text value column
        {{id title}} ... on BoardRelationValue {{display_value linked_items {{id}}}} 
        ... on MirrorValue {{display_value}}}}'''

        query = f'''{{boards(ids: {board_id}) {{groups(ids: "{group_id}") {{items_page(limit: 8 query_params: {{rules: 
        {{column_id: "{column_id}", compare_value: [{index_value}]}}}}) {{cursor items {{{item_attributes}}}}}}}}}}}'''

        data = {'query': query}
        response = self.client.send_post_request(data)

        # Handle API response
        if not response:
            return None
        try:
            items_page = response["data"]["boards"][0]["groups"][0]["items_page"]
            items = items_page["items"]
            cursor = items_page["cursor"]
            if cursor is None:
                return items
            all_items = self.client.get_next_page(cursor, items, item_attributes)
            return all_items
        except (TypeError, KeyError, IndexError):
            return response

    ####################################################################################################################
    def get_items_page_between_dates(self, board_id, group_id, columns, column_id, start_date, end_date):
        """
        Get Items page between a specified date range. Supports Timeline columns and start dates.
        :param board_id: UUID of the board the items are on.
        :param group_id: UUID of the group the items are in.
        :param columns: List of column IDs
        :param column_id: UUID of the timeline column to filter on.
        :param start_date: Start of date range.
        :param end_date: End of date range.
        :return: Array containing Item IDs. Return None or error message on API call failure.
        """
        column_string = ", ".join(f"\"{column}\"" for column in columns)
        item_attributes = f'''name id parent_item{{id name}} column_values(ids: [{column_string}]) {{text value column
        {{id title}} ... on BoardRelationValue {{display_value linked_items {{id}}}} ... on MirrorValue 
        {{display_value}}}}'''

        query = f'''{{boards(ids: {board_id}) {{groups(ids: "{group_id}") {{items_page(limit: 8 query_params: {{rules: 
        {{column_id: "{column_id}", compare_value: ["{start_date}", "{end_date}"], compare_attribute: "START_DATE", 
        operator: between}}}}) {{cursor items {{{item_attributes}}}}}}}}}}}'''
        data = {'query': query}
        response = self.client.send_post_request(data)

        # Handle API response
        if not response:
            return None
        try:
            items_page = response["data"]["boards"][0]["groups"][0]["items_page"]
            items = items_page["items"]
            cursor = items_page["cursor"]
            if cursor is None:
                return items
            all_items = self.client.get_next_page(cursor, items, item_attributes)
            return all_items
        except (TypeError, KeyError, IndexError):
            return response

    ####################################################################################################################
    def get_items_page_between_date(self, board_id, group_id, columns, column_id, start_date, end_date):
        """
        Get Items page between a specified date range. Supports Date columns and start dates.
        :param board_id: UUID of the board the items are on.
        :param group_id: UUID of the group the items are in.
        :param columns: List of column IDs.
        :param column_id: UUID of the date column to filter on.
        :param start_date: Start of date range.
        :param end_date: End of date range.
        :return: Array containing Item IDs. Return None or error message on API call failure.
        """
        column_string = ", ".join(f"\"{column}\"" for column in columns)
        item_attributes = f'''name id parent_item{{id name}} column_values(ids: [{column_string}]) {{text value column
        {{id title}} ... on BoardRelationValue {{display_value linked_items {{id}}}} ... on MirrorValue 
        {{display_value}}}}'''

        query = f'''{{boards(ids: {board_id}) {{groups(ids: "{group_id}") {{items_page(limit: 500 query_params: {{rules:
        {{column_id: "{column_id}", compare_value: ["{start_date}", "{end_date}"], operator: between}}}}) {{cursor items
        {{{item_attributes}}}}}}}}}}}'''

        data = {'query': query}
        response = self.client.send_post_request(data)

        # Handle API response
        if not response:
            return None
        try:
            items_page = response["data"]["boards"][0]["groups"][0]["items_page"]
            items = items_page["items"]
            cursor = items_page["cursor"]
            if cursor is None:
                return items
            all_items = self.client.get_next_page(cursor, items, item_attributes)
            return all_items
        except (TypeError, KeyError, IndexError):
            return response

    ####################################################################################################################
    def get_item_ids_between_dates(self, board_id, group_id, column_id, start_date, end_date):
        """
        Get Item IDs between a specified date range. Supports Timeline columns and start dates.
        :param board_id: UUID of the board the items are on.
        :param group_id: UUID of the group the items are in.
        :param column_id: UUID of the timeline column to filter on.
        :param start_date: Start of date range.
        :param end_date: End of date range.
        :return: Array containing Item IDs. Return None or error message on API call failure.
        """
        item_attributes = "id"
        query = f'''{{boards(ids: {board_id}) {{groups(ids: "{group_id}") {{items_page(limit: 500, 
            query_params: {{rules: {{column_id: "{column_id}", compare_value: ["{start_date}", "{end_date}"], 
            compare_attribute: "START_DATE", operator: between}}}}) {{cursor items {{{item_attributes}}}}}}}}}}}'''

        data = {'query': query}
        response = self.client.send_post_request(data)

        # Handle API response
        if not response:
            return None
        try:
            items = response['data']['boards'][0]['groups'][0]['items_page']['items']
            cursor = response['data']['boards'][0]['groups'][0]['items_page']['cursor']
            item_ids = [item['id'] for item in items]
            if cursor is None:
                return item_ids
            all_item_ids = self.client.get_next_page(cursor, items, item_attributes)
            return [int(item['id']) for item in all_item_ids]
        except (TypeError, KeyError, IndexError):
            return response

    ####################################################################################################################
    def get_item_ids_between_date(self, board_id, group_id, column_id, start_date, end_date):
        """
        Get Item IDs between a specified date range. Supports Date columns.
        :param board_id: UUID of the board the items are on.
        :param group_id: UUID of the group the items are in.
        :param column_id: UUID of the date column to filter on.
        :param start_date: Start of date range.
        :param end_date: End of date range.
        :return: Array containing Item IDs. Return None or error message on API call failure.
        """
        item_attributes = "id"
        query = f'''{{boards(ids: {board_id}) {{groups(ids: "{group_id}") {{items_page(limit: 500, 
            query_params: {{rules: {{column_id: "{column_id}", compare_value: ["{start_date}", "{end_date}"], 
            operator: between}}}}) {{cursor items {{{item_attributes}}}}}}}}}}}'''

        data = {'query': query}
        response = self.client.send_post_request(data)

        # Handle API response
        if not response:
            return None
        try:
            items = response['data']['boards'][0]['groups'][0]['items_page']['items']
            cursor = response['data']['boards'][0]['groups'][0]['items_page']['cursor']
            item_ids = [item['id'] for item in items]
            if cursor is None:
                return item_ids
            all_item_ids = self.client.get_next_page(cursor, items, item_attributes)
            return [int(item['id']) for item in all_item_ids]
        except (TypeError, KeyError, IndexError):
            return response

    ####################################################################################################################
    def get_item_columns(self, item_list, column_id_list):
        """
        Get column values of items on a board.
        :param item_list: Array of item IDs to query. Item IDs must be submitted as integers.
        :param column_id_list: Array of column UUIDs. Column IDs must be submitted as strings.
        :return: Array of dictionaries containing item and column values. Return None or error message on API call failure.
        """
        asyncio.run(self.client.column_task_handler(item_list, column_id_list))

        current_results = self.client.results
        self.client.results = []

        # Handle API response
        if not current_results:
            return None
        try:
            return [item['data']['items'][0] for item in current_results]
        except (TypeError, KeyError, IndexError):
            return current_results


    ####################################################################################################################
    def create_item(self, board, group, item_name):
        """
        Create a new item within a group on a specific board.
        :param board: UUID of the board to create the item on.
        :param group: UUID of the group to create the item in.
        :param item_name: Name of the new item to be created.
        :return: UUID of the newly created item. Return None or error message on API call failure.
        """
        query = f'''mutation {{create_item (board_id: {board}, group_id: "{group}", item_name: "{item_name}") {{id}}}}'''
        data = {'query': query}
        response = self.client.send_post_request(data)

        # Handle API response
        if not response:
            return None
        try:
            return response['data']['create_item']['id']
        except (TypeError, KeyError, IndexError):
            return response

    ####################################################################################################################
    def create_item_with_column_values(self, board, group, item_name, column_dict):
        """
        Create a new item within a group on a board and populate specific column values on item creation.
        :param board: UUID of the board to create the item on.
        :param group: UUID of the group to create the item in.
        :param item_name: Name of the new item.
        :param column_dict: Dictionary of column values to set when item is created.
        Example: {"text": {"type": "text", "values": ["Hello World"]}}
        :return: UUID of the newly created item. Return None or error message on API call failure.
        """
        column_string = self.client.column_id_formatter(column_dict)
        query = f'''mutation {{create_item (board_id: {board}, group_id: "{group}", item_name: "{item_name}", 
            column_values: "{column_string}") {{id}}}}'''
        data = {'query': query}
        response = self.client.send_post_request(data)

        # Handle API response
        if not response:
            return None
        try:
            return response['data']['create_item']['id']
        except (TypeError, KeyError, TypeError):
            return response

    ####################################################################################################################
    def delete_item(self, item_id):
        """
        Delete an item from the platform.
        :param item_id: UUID of the item to delete.
        :return: Confirmation message or error message.
        """
        query = f'''mutation {{delete_item (item_id: {item_id}) {{id}}}}'''
        data = {"query": query}
        response = self.client.send_post_request(data)

        # Handle API response
        if not response:
            return None
        try:
            if response['data']['delete_item']['id']:
                return True
            else:
                return response
        except (TypeError, KeyError, IndexError):
            return response

    ####################################################################################################################
    # New - get_items_by_column_values
    def get_items_by_column_values(self, board_id, column_filters, limit=100):
        """
        Retrieve all items from a board that match specific column values using pagination.
        :param board_id: The ID of the board to retrieve items from.
        :param column_filters: List of dictionaries with keys 'column_id' and 'column_values' to filter items
        (for supported columns see https://developer.monday.com/api-reference/reference/items-page-by-column-values).
        :param limit: The number of items to return per page (maximum 500). Defaults to 100.
        :return: List of all items that match the specified filters. Return None or error message on API call failure.
        """

        # Ensure column_values are formatted correctly for GraphQL
        def format_column_values(values):
            return ", ".join(f'"{v}"' for v in values)  # Ensure double quotes for each value

        # Prepare column filters for GraphQL syntax
        columns_str = ', '.join([f'{{column_id: "{col_filter["column_id"]}", '
                f'column_values: [{format_column_values(col_filter["column_values"])}]}}'
                for col_filter in column_filters])

        # Initial request with columns
        query = f'''query {{items_page_by_column_values(board_id: {board_id}, columns: [{columns_str}], limit: {limit}) 
        {{cursor items {{ id name column_values {{ id text }} }}}}}}'''

        payload = {"query": query}
        response = self.client.send_post_request(payload)

        # Handle API response
        if not response:
            return None
        try:
            # Extract initial items and cursor
            next_data = response["data"]["items_page_by_column_values"]
            all_items = next_data["items"]
            cursor = next_data.get("cursor")

            # Fetch remaining pages if cursor exists
            while cursor:
                query = f'''query {{items_page_by_column_values(board_id: {board_id}, limit: {limit}, 
                cursor: "{cursor}") {{cursor items {{ id name column_values {{ id text }}}}}}}}'''
                payload = {"query": query}
                response = self.client.send_post_request(payload)

                if not response:
                    return all_items  # Return collected items if no response
                try:
                    next_data = response["data"]["items_page_by_column_values"]
                    all_items.extend(next_data["items"])
                    cursor = next_data.get("cursor")
                except (KeyError, TypeError, IndexError):
                    break

            return all_items
        except (TypeError, KeyError, IndexError):
            return response

    ####################################################################################################################
    # New - archive_item
    def archive_item(self, item_id):
        """
        Archive an item on a board.
        :param item_id: The unique identifier of the item to archive.
        :return: JSON response confirming the archiving of the item. Return None or error message on API call failure.
        """
        mutation = f'''mutation {{archive_item (item_id: {item_id}) {{id}}}}'''
        payload = {'query': mutation}
        response = self.client.send_post_request(payload)

        # Handle API response
        if not response:
            return None
        try:
            if response['data']['archive_item']['id']:
                return True
            else:
                return response
        except (TypeError, KeyError, IndexError):
            return response

    ####################################################################################################################
    # New - duplicate_item
    def duplicate_item(self, board_id, item_id, with_updates=True):
        """
        Duplicate an item, including its updates.
        :param board_id: The unique identifier of the board to duplicate the item in.
        :param item_id: The unique identifier of the item to duplicate.
        :param with_updates: Boolean indicating whether to include updates in the duplication.
        :return: UUID of the duplicated item. Return None or error message on API call failure.
        """

        mutation = f'''mutation {{duplicate_item (board_id: {board_id}, item_id: {item_id}, with_updates: 
            {str(with_updates).lower()}) {{id}}}}'''
        payload = {'query': mutation}
        response = self.client.send_post_request(payload)

        # Handle API response
        if not response:
            return None
        try:
            return response['data']['duplicate_item']['id']
        except (TypeError, KeyError, IndexError):
            return response

    ####################################################################################################################
    # New - move_item_to_board
    def move_item_to_board(self, item_id, target_board_id, target_group_id, column_mapping=None):
        """
        Move an item to a different board with optional column mapping.
        :param item_id: ID of the item to move.
        :param target_board_id: ID of the target board.
        :param target_group_id: ID of the target group within the target board.
        :param column_mapping: Optional list of dictionaries mapping source columns to target columns.
        :return: UUID of the moved item. Return None or error message on API call failure.
        """
        # Construct the column mapping argument if provided
        if column_mapping:
            columns_mapping = ', '.join(
                f'{{source: "{source}", target: "{target}"}}' for source, target in column_mapping.items())
        else:
            columns_mapping = ''

        mutation = f'''mutation {{move_item_to_board (board_id: {target_board_id}, group_id: "{target_group_id}",
                item_id: {item_id}, columns_mapping: [{columns_mapping}]) {{id}}}}'''
        payload = {'query': mutation}
        response = self.client.send_post_request(payload)

        # Handle API response
        if not response:
            return None
        try:
            return response['data']['move_item_to_board']['id']
        except (TypeError, KeyError, IndexError):
            return response

    ####################################################################################################################


from setuptools import setup, find_packages

# Read the contents of README.md
with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setup(
    name='pymonday',
    version='3.0.2',
    packages=find_packages(include=['pymonday', 'pymonday.*']),
    include_package_data=True,
    description='pyMonday is a monday.com API Python Client Library, compatible with API version 2025-01 and later.',
    long_description = long_description,
    long_description_content_type="text/markdown",
    install_requires=['httpx', 'python-dotenv', 'PyYAML', 'asyncio'],
)


# pymonday/methods/folders.py

########################################################################################################################
# FOLDERS CLASS
########################################################################################################################
class Folders:
    def __init__(self, client):
        self.client = client

    ####################################################################################################################
    def get_folders(self, workspace_id):
        """
        Get all folders in a workspace.
        :param workspace_id: UUID of the workspace.
        :return: Array of Dictionaries containing folder data. Return None or error message on API call failure.
        """
        query_string = f'''query {{folders (workspace_ids: {workspace_id}) {{name id children {{id name}}}}}}'''
        data = {'query': query_string}
        response = self.client.send_post_request(data)

        # Handle API response
        if not response:
            return None
        try:
            return response['data']['folders']
        except (TypeError, KeyError, IndexError):
            return response

    ####################################################################################################################
    def create_folder(self, name, workspace_id, **kwargs):
        """
        Create a folder in a workspace. Pass keyword arguments to configure folder attributes. **Keywords must be passed
        exactly as defined below. Folder colors can be found here:
        https://asset.cloudinary.com/monday-platform-dev/3e39afb2309b512f4f53cc9173554d48
        :param name: The Folders name (Required)
        :param workspace_id: The unique identifier of the workspace to create the new folder in (Required)
        :param kwargs:
            color: The Folders color
            parent_folder_id: The ID of the folder you want to nest the new one under. Nesting is limited to 1 Tier.
        :return: UUID of the newly created folder. Return None or error message on API call failure.
        """

        folder_name = f"\"{name}\""
        arg_string = f'name: {folder_name}, workspace_id: {workspace_id}, '
        for key, value in kwargs.items():
            arg_string = arg_string + f"{key}: {value}, "
        arg_string = arg_string[:-2]
        query_string = f'''mutation {{create_folder ({arg_string}) {{id}}}}'''
        data = {'query': query_string}
        response = self.client.send_post_request(data)

        # Handle API response
        if not response:
            return None
        try:
            return response['data']['create_folder']['id']
        except (TypeError, KeyError, IndexError):
            return response

    ####################################################################################################################
    def update_folder(self, folder_id, **kwargs):
        """
        Update a folder in a workspace. Pass keyword arguments to configure folder attributes. **Keywords must be passed
        exactly as defined below. Folder colors can be found here:
        https://asset.cloudinary.com/monday-platform-dev/3e39afb2309b512f4f53cc9173554d48
        :param folder_id: UUID of the folder to update. (Required)
        :param kwargs:
            name: Updated name of the folder
            color: Updated folder color
            parent_folder_id: The ID of the folder you want to nest the updated one under.
        :return: UUID of the updated folder. Return None or error message on API call failure.
        """
        if 'name' in kwargs.keys():
            kwargs['name'] = f"\"{kwargs['name']}\""
        arg_string = f"folder_id: {folder_id}, "
        for key, value in kwargs.items():
            arg_string = arg_string + f"{key}: {value}, "
        arg_string = arg_string[:-2]
        query_string = f'''mutation {{update_folder ({arg_string}) {{id}}}}'''
        data = {'query': query_string}
        response = self.client.send_post_request(data)

        # Handle API response
        if not response:
            return None
        try:
            return response['data']['update_folder']['id']
        except (TypeError, KeyError, IndexError):
            return response

    ####################################################################################################################
    def delete_folder(self, folder_id):
        """
        Delete a folder from a workspace
        :param folder_id: UUID of the folder to delete
        :return: True if successful. Return None or error message on API call failure
        """
        query_string = f'''mutation {{delete_folder (folder_id: {folder_id}) {{id}}}}'''
        data = {'query': query_string}
        response = self.client.send_post_request(data)

        # Handle API response
        if not response:
            return None
        try:
            if response['data']['delete_folder']['id']:
                return True
        except (TypeError, KeyError, IndexError):
            return response

    ####################################################################################################################


# pymonday/methods/__init__.py

########################################################################################################################
# IMPORTS
########################################################################################################################
from .account import Account
from .activity_logs import ActivityLogs
from .boards import Boards
from .columns import Columns
from .docs import Docs
from .doc_blocks import DocBlocks
from .files import Files
from .folders import Folders
from .groups import Groups
from .items import Items
from .notifications import Notifications
from .subitems import Subitems
from .tags import Tags
from .teams import Teams
from .updates import Updates
from .users import Users
from .workspaces import Workspaces

__all__ = [
    "Account",
    "ActivityLogs",
    "Boards",
    "Columns",
    "Docs",
    "DocBlocks",
    "Files",
    "Folders",
    "Groups",
    "Items",
    "Notifications",
    "Subitems",
    "Tags",
    "Teams",
    "Updates",
    "Users",
    "Workspaces",
]



# pymonday/methods/notifications.py

########################################################################################################################
# NOTIFICATIONS CLASS
########################################################################################################################
class Notifications:
    def __init__(self, client):
        self.client = client

    ####################################################################################################################
    def create_notification(self, user_id, target, body, target_type):
        """
        Send a notification to a user.
        :param user_id: UUID of the user to send the notification to.
        :param target: The target's unique identifier. The value depends on the target_type
        :param body: The notification's text.
        :param target_type: the target's type: Project or Post.
            - Project: sends a notification referring to a specific item or board
            - Post : sends a notification referring to a specific item's update or reply
        :return: Text of the notification. Return None or error message on API call failure.
        """
        query_string = f'''mutation {{create_notification 
        (user_id: {user_id}, target_id: {target}, text: "{body}", target_type: {target_type}) {{text}}}}'''
        data = {'query': query_string}
        response = self.client.send_post_request(data)

        # Handle API response
        if not response:
            return None
        try:
            return response['data']['create_notification']['text']
        except (TypeError, KeyError, IndexError):
            return response

    ####################################################################################################################